from .state_machine import StateMachine


class AWSResourceDriver:
    def __init__(self, cloudformation_stack, boto_session):
        self.__cloudformation_stack = cloudformation_stack
        self.__boto_session = boto_session

    def get_state_machine(self, fully_qualified_cfn_logical_resource_id):
        state_machine_arn = self.__cloudformation_stack.get_physical_resource_id_for(
            fully_qualified_cfn_logical_resource_id
        )
        return StateMachine(state_machine_arn, self.__boto_session)



import boto3

from .aws_resource_driver import AWSResourceDriver
from .aws_resource_mocking_engine import AWSResourceMockingEngine
from .aws_test_double_driver import AWSTestDoubleDriver
from .boto_session_factory import BotoSessionFactory
from .cloudformation_stack import CloudFormationStack


class TestResourcesFactory:
    def __init__(self, cloudformation_stack_name: str, aws_profile: str):
        self.cloudformation_stack_name = cloudformation_stack_name
        self.aws_profile = aws_profile
        self.__initialised = False

    @property
    def resource_driver(self) -> AWSResourceDriver:
        self.__ensure_initialised()
        return self.__aws_resource_driver

    @property
    def mocking_engine(self) -> AWSResourceMockingEngine:
        self.__ensure_initialised()
        return self.__mocking_engine

    @property
    def test_double_driver(self) -> AWSTestDoubleDriver:
        self.__ensure_initialised()
        return self.__test_double_driver

    def __ensure_initialised(self):
        if not self.__initialised:
            self.__initialised = True

            developer_boto_session = boto3.session.Session(profile_name=self.aws_profile)

            boto_session_factory = BotoSessionFactory(developer_boto_session)

            cloudformation_stack = CloudFormationStack(self.cloudformation_stack_name, developer_boto_session)

            self.__aws_resource_driver = AWSResourceDriver(
                cloudformation_stack,
                boto_session_factory.create_boto_session_with_assumed_role(
                    cloudformation_stack.get_physical_resource_id_for("AWSTestHarnessTesterRole::Role")
                )
            )

            test_doubles_stack = CloudFormationStack(
                cloudformation_stack.get_physical_resource_id_for('AWSTestHarnessTestDoubles'),
                developer_boto_session
            )

            test_double_manager_boto_session = boto_session_factory.create_boto_session_with_assumed_role(
                test_doubles_stack.get_physical_resource_id_for("TestDoubleManagerRole")
            )

            self.__test_double_driver = AWSTestDoubleDriver(test_doubles_stack, test_double_manager_boto_session)
            self.__mocking_engine = AWSResourceMockingEngine(self.__test_double_driver,
                                                             test_double_manager_boto_session)



from typing import Callable, Dict, List
from unittest.mock import Mock, create_autospec
from uuid import uuid4

from boto3 import Session

from .a_state_machine_execution_failure import AStateMachineExecutionFailure
from .a_thrown_exception import AThrownException
from .exit_code import ExitCode
from .aws_test_double_driver import AWSTestDoubleDriver
from .message_listener import MessageListener
from .task_context import TaskContext


class AWSResourceMockingEngine:
    __mocking_session_id: str = None
    __message_listener: MessageListener = None

    def __init__(self, test_double_driver: AWSTestDoubleDriver, boto_session: Session):
        self.__mock_event_handlers: Dict[str, Mock] = {}
        self.__test_double_driver = test_double_driver
        self.__boto_session = boto_session

    def reset(self):
        if self.__message_listener:
            self.__message_listener.stop()

        self.__set_mocking_session_id()

        self.__message_listener = MessageListener(self.__test_double_driver, self.__boto_session,
                                                  lambda: self.__mocking_session_id)

        self.__message_listener.start()

    def mock_a_lambda_function(self, function_id: str,
                               event_handler: Callable[[Dict[str, any]], Dict[str, any] | AThrownException]) -> Mock:
        def lambda_handler(_: Dict[str, any]) -> Dict[str, any] | AThrownException:
            pass

        mock_event_handler: Mock = create_autospec(lambda_handler, name=function_id)
        mock_event_handler.side_effect = event_handler

        self.__message_listener.register_lambda_function_event_handler(
            self.__test_double_driver.get_lambda_function_name(function_id),
            mock_event_handler
        )

        mock_id = self.__get_lambda_function_mock_id(function_id)
        self.__mock_event_handlers[mock_id] = mock_event_handler

        return mock_event_handler

    def mock_a_state_machine(self, state_machine_id,
                             handle_execution_input: Callable[
                                 [Dict[str, any]], Dict[str, any] | AStateMachineExecutionFailure]) -> Mock:
        def execution_input_handler(_: Dict[str, any]) -> Dict[str, any] | AStateMachineExecutionFailure:
            pass

        mock: Mock = create_autospec(execution_input_handler, name=state_machine_id)
        mock.side_effect = handle_execution_input

        self.__message_listener.register_state_machine_execution_input_handler(
            self.__test_double_driver.get_state_machine_arn(state_machine_id),
            mock
        )

        mock_id = self.__get_state_machine_mock_id(state_machine_id)
        self.__mock_event_handlers[mock_id] = mock

        return mock

    def mock_an_ecs_task_container(self, task: str, container: str,
                                   handler: Callable[[TaskContext], ExitCode]) -> Mock:
        def ecs_task_handler(_: List[str]) -> ExitCode:
            pass

        mock: Mock = create_autospec(ecs_task_handler, name=task)
        mock.side_effect = handler

        task_definition_arn = self.__test_double_driver.get_task_definition_arn(task)
        self.__message_listener.register_ecs_task_container_handler(task_definition_arn, container, mock)

        mock_id = self.__get_ecs_task_container_mock_id(task, container)
        self.__mock_event_handlers[mock_id] = mock

        return mock

    def __set_mocking_session_id(self) -> str:
        self.__mocking_session_id = str(uuid4())
        self.__test_double_driver.test_context_bucket.put_object('test-id', self.__mocking_session_id)
        return self.__mocking_session_id

    def get_mock_lambda_function(self, function_id: str) -> Mock:
        mock_id = self.__get_lambda_function_mock_id(function_id)
        return self.__mock_event_handlers[mock_id]

    def get_mock_state_machine(self, state_machine_id: str) -> Mock:
        mock_id = self.__get_state_machine_mock_id(state_machine_id)
        return self.__mock_event_handlers[mock_id]

    def get_mock_ecs_task_container(self, task: str, container: str) -> Mock:
        mock_id = self.__get_ecs_task_container_mock_id(task, container)
        return self.__mock_event_handlers[mock_id]

    @staticmethod
    def __get_lambda_function_mock_id(state_machine_id: str) -> str:
        return f'LambdaFunction::{state_machine_id}'

    @staticmethod
    def __get_state_machine_mock_id(state_machine_id: str) -> str:
        return f'StateMachine::{state_machine_id}'

    @staticmethod
    def __get_ecs_task_container_mock_id(task_family: str, container: str) -> str:
        return f'ECSTask::{task_family}#{container}'



from uuid import uuid4

import boto3
from boto3 import Session
from mypy_boto3_sts import STSClient


class BotoSessionFactory:

    def __init__(self, boto_session: Session):
        super().__init__()
        self.__boto_session = boto_session

    def create_boto_session_with_assumed_role(self, assumed_role_name: str):
        sts_client: STSClient = self.__boto_session.client("sts")

        identity = sts_client.get_caller_identity()
        aws_account_id = identity["Account"]

        assume_role_response = sts_client.assume_role(
            RoleArn=f'arn:aws:iam::{aws_account_id}:role/{assumed_role_name}',
            RoleSessionName=str(uuid4())
        )

        assumed_role_credentials = assume_role_response["Credentials"]

        return boto3.session.Session(
            aws_access_key_id=assumed_role_credentials["AccessKeyId"],
            aws_secret_access_key=assumed_role_credentials["SecretAccessKey"],
            aws_session_token=assumed_role_credentials["SessionToken"],
            region_name=self.__boto_session.region_name
        )



import json
from datetime import datetime
from enum import StrEnum
from time import sleep
from typing import Optional

from mypy_boto3_stepfunctions import SFNClient


class StateMachineExecutionState(StrEnum):
    RUNNING = "RUNNING"
    ABORTED = "ABORTED"
    TIMED_OUT = "TIMED_OUT"
    FAILED = "FAILED"
    SUCCEEDED = "SUCCEEDED"


FINAL_STATES = [
    StateMachineExecutionState.SUCCEEDED,
    StateMachineExecutionState.FAILED,
    StateMachineExecutionState.TIMED_OUT,
    StateMachineExecutionState.ABORTED
]


class StateMachineExecution:
    __status: Optional[StateMachineExecutionState]
    __output: Optional[str]
    __cause: Optional[str]
    __error: Optional[str]

    def __init__(self, execution_arn: str, sfn_client: SFNClient):
        self.__execution_arn = execution_arn
        self.__sfn_client = sfn_client
        self.__status = None

    def assert_succeeded(self):
        assert self.succeeded, (
            f"Execution failed with error '{self.failure_error}' and cause: '{self.failure_cause}'."
            if self.failed else f"Execution did not succeed. Current status is '{self.status}'."
        )

    @property
    def name(self):
        return self.__sfn_client.describe_execution(executionArn=self.__execution_arn)["name"]

    @property
    def running(self):
        return self.status == StateMachineExecutionState.RUNNING

    @property
    def succeeded(self):
        return self.status == StateMachineExecutionState.SUCCEEDED

    @property
    def failed(self):
        return self.status == StateMachineExecutionState.FAILED

    @property
    def timed_out(self):
        return self.status == StateMachineExecutionState.TIMED_OUT

    @property
    def aborted(self):
        return self.status == StateMachineExecutionState.ABORTED

    @property
    def status(self):
        if self.__status not in FINAL_STATES:
            self.__retrieve_status()

        return self.__status

    @property
    def output_json(self):
        return json.loads(self.output)

    @property
    def output(self):
        if self.__status != StateMachineExecutionState.SUCCEEDED:
            raise Exception(
                f"State machine output unavailable because execution is currently in a '{self.__status}' state")

        return self.__output

    @property
    def failure_cause(self):
        if self.__status != StateMachineExecutionState.FAILED:
            raise Exception(
                f"State machine failure cause is unavailable because execution is currently in a '{self.__status}' state")

        return self.__cause

    @property
    def failure_error(self):
        if self.__status != StateMachineExecutionState.FAILED:
            raise Exception(
                f"State machine failure error is unavailable because execution is currently in a '{self.__status}' state")

        return self.__error

    def wait_for_completion(self, timeout_seconds: float = 30):
        start_time = datetime.now()

        while self.status == StateMachineExecutionState.RUNNING:
            wait_time = datetime.now() - start_time

            if wait_time.total_seconds() >= timeout_seconds:
                raise Exception(f"State machine execution timed out after {timeout_seconds} seconds")

            sleep(0.5)

    def __retrieve_status(self):
        response = self.__sfn_client.describe_execution(executionArn=self.__execution_arn)

        self.__status = StateMachineExecutionState(response["status"])

        if self.__status == StateMachineExecutionState.SUCCEEDED:
            self.__output = response.get("output")

        if self.__status == StateMachineExecutionState.FAILED:
            self.__cause = response.get("cause")
            self.__error = response.get("error")



import json
import sys
import traceback
from datetime import datetime, timedelta
from threading import Thread
from typing import Dict, Callable, Any, cast, List

from boto3 import Session
from botocore.exceptions import ClientError
from mypy_boto3_dynamodb.service_resource import DynamoDBServiceResource
from mypy_boto3_sqs import SQSClient

from .a_state_machine_execution_failure import AStateMachineExecutionFailure
from .a_thrown_exception import AThrownException
from .exit_code import ExitCode
from .aws_test_double_driver import AWSTestDoubleDriver
from .task_context import TaskContext


class MessageListener(Thread):
    __event_handlers: Dict[str, Callable[..., Any]] = {}
    __stop_waiting: bool = False

    def __init__(self, test_double_driver: AWSTestDoubleDriver, boto_session: Session,
                 get_mocking_session_id: Callable[[], str]):
        super().__init__(daemon=True)
        self.__sqs_client: SQSClient = boto_session.client('sqs')
        self.__dynamodb_resource: DynamoDBServiceResource = boto_session.resource('dynamodb')
        self.__test_double_driver = test_double_driver
        self.__get_mocking_session_id = get_mocking_session_id

    def run(self):
        # noinspection PyBroadException
        try:
            while True:
                print('Waiting for message...')
                result = self.__sqs_client.receive_message(
                    QueueUrl=self.__test_double_driver.events_queue_url,
                    AttributeNames=['All'],
                    MessageAttributeNames=['All'],
                    MaxNumberOfMessages=1,
                    WaitTimeSeconds=20,
                )

                if 'Messages' in result:
                    mocking_session_id = self.__get_mocking_session_id()
                    print(f'Current mocking session id: {mocking_session_id}')

                    for message in result['Messages']:
                        print(f'Message received: {json.dumps(message)}')

                        # Delete message before processing to prevent other consumers from processing it when the visibility timeout expires
                        # This is necessary in case processing involves a long running operation, e.g. a sleep to control concurrency
                        # Additionally, messages should be deleted regardless of their mocking session ID, to avoid poison pills from previous test runs
                        try:
                            receipt_handle = message['ReceiptHandle']
                            self.__sqs_client.delete_message(
                                QueueUrl=self.__test_double_driver.events_queue_url,
                                ReceiptHandle=receipt_handle
                            )
                        except ClientError as e:
                            print(f"Failed to delete message: {e}")

                        if message['MessageAttributes']['MockingSessionId']['StringValue'] == mocking_session_id:
                            message_consumer_thread = Thread(
                                daemon=True,
                                target=self.__consume_message,
                                args=(message,)
                            )

                            message_consumer_thread.start()

                else:
                    print('No messages received')

                if self.__stop_waiting:
                    print('Stopped waiting for messages')
        except Exception:
            print('Exception thrown whilst waiting for messages')
            exc_type, exc_value, exc_traceback = sys.exc_info()
            print(f"Exception Type: {exc_type.__name__}")
            print(f"Exception Message: {exc_value}")
            traceback.print_tb(exc_traceback)

    def stop(self):
        self.__stop_waiting = True

    def register_lambda_function_event_handler(self, function_name: str, event_handler):
        handler_id = self.__get_lambda_function_event_handler_id(function_name)
        self.__event_handlers[handler_id] = event_handler

    def register_state_machine_execution_input_handler(
            self,
            state_machine_arn: str,
            execution_input_handler: Callable[[Dict[str, any]], Dict[str, any]]
    ):
        handler_id = self.__get_state_machine_execution_input_handler_id(state_machine_arn)
        self.__event_handlers[handler_id] = execution_input_handler

    def register_ecs_task_container_handler(self, task_definition_arn: str, container_name: str, task_handler: Callable[[List[str]], ExitCode]):
        handler_id = self.__get_ecs_task_container_handler_id(task_definition_arn, container_name)
        self.__event_handlers[handler_id] = task_handler

    def __consume_message(self, message: Dict[str, Any]) -> None:
        event_message_payload = json.loads(message['Body'])

        default_invocation_type = dict(StringValue='Lambda Function Invocation')
        invocation_type = message['MessageAttributes'].get('InvocationType', default_invocation_type)['StringValue']

        if invocation_type == 'State Machine Execution':
            item = self.__generate_result_record_for_state_machine_execution(event_message_payload)
        elif invocation_type == 'Lambda Function Invocation':
            item = self.__generate_result_record_for_lamdba_function_invocation(event_message_payload)
        elif invocation_type == 'ECS Task Execution':
            item = self.__generate_result_record_for_ecs_task_container_execution(event_message_payload)
        else:
            raise Exception(f'Unknown invocation type: "{invocation_type}"')

        self.__dynamodb_resource.Table(self.__test_double_driver.results_table_name).put_item(Item=item)

    def __generate_result_record_for_lamdba_function_invocation(self, event_message_payload: Dict[str, Any]) -> Dict[
        str, Any]:
        function_event = json.loads(event_message_payload['event'])
        function_invocation_id = event_message_payload['invocationId']
        function_name = event_message_payload['functionName']

        print(f"{function_name} invocation with invocation ID {function_invocation_id} "
              f"received event {function_event}")

        handler_id = self.__get_lambda_function_event_handler_id(function_name)
        event_handler = self.__event_handlers[handler_id]
        event_handler_result = event_handler(function_event)
        assert isinstance(event_handler_result, dict) or isinstance(event_handler_result, AThrownException)

        result: Dict[str, Any] = dict(raiseException=False)

        if isinstance(event_handler_result, AThrownException):
            function_result = cast(AThrownException, event_handler_result)
            result['raiseException'] = True
            result['exceptionMessage'] = function_result.message
        else:
            function_result = cast(Dict[str, Any], event_handler_result)
            result['payload'] = json.dumps(function_result)

        print(f'Returning result: {json.dumps(result)}')

        return dict(
            partitionKey=f'{function_name}#{function_invocation_id}',
            result=result,
            functionName=function_name,
            invocationId=function_invocation_id,
            functionEvent=function_event,
            ttl=int((datetime.now() + timedelta(hours=12)).timestamp())
        )

    def __generate_result_record_for_state_machine_execution(self, event_message_payload: Dict[str, Any]) -> Dict[
        str, Any]:
        execution_input = event_message_payload['input']
        invocation_id = event_message_payload['invocationId']
        state_machine_arn = event_message_payload['stateMachineArn']

        print(f"{state_machine_arn} execution with execution ID {invocation_id} "
              f"received input {execution_input}")

        handler_id = self.__get_state_machine_execution_input_handler_id(state_machine_arn)
        execution_input_handler = self.__event_handlers[handler_id]
        handler_result = execution_input_handler(execution_input)
        assert isinstance(handler_result, dict) or isinstance(handler_result, AStateMachineExecutionFailure)

        result: Dict[str, Any] = dict(failExecution=False)

        if isinstance(handler_result, AStateMachineExecutionFailure):
            state_machine_result = cast(AStateMachineExecutionFailure, handler_result)
            result['failExecution'] = True
            result['error'] = state_machine_result.error
            result['cause'] = state_machine_result.cause
        else:
            state_machine_result = cast(Dict[str, Any], handler_result)
            result['payload'] = json.dumps(state_machine_result)

        print(f'Returning result: {json.dumps(result)}')

        return dict(
            partitionKey=f'{state_machine_arn}#{invocation_id}',
            result=result,
            stateMachineArn=state_machine_arn,
            invocationId=invocation_id,
            input=execution_input,
            ttl=int((datetime.now() + timedelta(hours=12)).timestamp())
        )

    def __generate_result_record_for_ecs_task_container_execution(self, event_message_payload: Dict[str, Any]) -> Dict[str, Any]:
        task_context_data = event_message_payload['taskContext']
        invocation_id = event_message_payload['invocationId']
        task_definition_arn = event_message_payload['taskDefinitionArn']
        container_name = event_message_payload['containerName']

        print(f"{task_definition_arn} task ({container_name} container) execution with invocation ID {invocation_id} "
              f"received task context {task_context_data}")

        handler_id = self.__get_ecs_task_container_handler_id(task_definition_arn, container_name)
        handler = self.__event_handlers[handler_id]

        task_context = TaskContext(
            task_context_data['commandArgs'],
            task_context_data['environmentVariables']
        )
        task_result = handler(task_context)
        assert isinstance(task_result, ExitCode)
        exit_code = cast(ExitCode, task_result)

        result: Dict[str, Any] = dict(exitCode=exit_code.value)
        print(f'Returning result: {json.dumps(result)}')

        return dict(
            partitionKey=f'{task_definition_arn}#{container_name}#{invocation_id}',
            result=result,
            taskDefinitionArn=task_definition_arn,
            containerName=container_name,
            invocationId=invocation_id,
            ttl=int((datetime.now() + timedelta(hours=12)).timestamp())
        )

    @staticmethod
    def __get_lambda_function_event_handler_id(function_name: str) -> str:
        return f'LambdaFunction::{function_name}'

    @staticmethod
    def __get_state_machine_execution_input_handler_id(state_machine_arn: str) -> str:
        return f'StateMachine::{state_machine_arn}'

    @staticmethod
    def __get_ecs_task_container_handler_id(task_definition_arn: str, container_name: str) -> str:
        return f'ECSTask::{task_definition_arn}#{container_name}'



from typing import Optional, List

from boto3 import Session
from mypy_boto3_s3.service_resource import S3ServiceResource, Bucket


class S3Bucket:

    def __init__(self, name: str, session: Session):
        super().__init__()

        self.__name = name
        s3_resource: S3ServiceResource = session.resource('s3')
        self.__boto_bucket_resource: Bucket = s3_resource.Bucket(name)

    def put_object(self, key: str, content: str):
        self.__boto_bucket_resource.put_object(Key=key, Body=content)

    def get_object(self, key: str) -> str:
        return self.__boto_bucket_resource.Object(key).get()['Body'].read().decode('utf-8')

    def list_objects(self, prefix: Optional[str] = None) -> List[str]:
        return [x.key for x in self.__boto_bucket_resource.objects.filter(**(dict(Prefix=prefix) if prefix else {}))]

    @property
    def name(self) -> str:
        return self.__name



from boto3 import Session

from aws_test_harness.cloudformation_stack import CloudFormationStack
from aws_test_harness.dynamodb_table import DynamoDBTable
from aws_test_harness.s3_bucket import S3Bucket


class AWSTestDoubleDriver:
    __test_context_bucket: S3Bucket = None
    __events_queue_url: str = None
    __results_table_name: str = None

    def __init__(self, cloudformation_stack: CloudFormationStack, boto_session: Session):
        self.__cloudformation_stack = cloudformation_stack
        self.__boto_session = boto_session

    def get_s3_bucket(self, bucket_id) -> S3Bucket:
        s3_bucket_name = self.__cloudformation_stack.get_physical_resource_id_for(
            f'{bucket_id}Bucket'
        )

        return S3Bucket(s3_bucket_name, self.__boto_session)

    def get_dynamodb_table(self, table_name) -> DynamoDBTable:
        ddb_table_name = self.__cloudformation_stack.get_physical_resource_id_for(
            f'{table_name}Table'
        )

        return DynamoDBTable(ddb_table_name, self.__boto_session)

    def get_lambda_function_name(self, function_id):
        return self.__cloudformation_stack.get_physical_resource_id_for(f'{function_id}Function')

    def get_state_machine_arn(self, state_machine_id):
        return self.__cloudformation_stack.get_physical_resource_id_for(f'{state_machine_id}StateMachine')

    def get_task_definition_arn(self, task_family):
        return self.__cloudformation_stack.get_physical_resource_id_for(f'{task_family}TaskDefinition')

    @property
    def events_queue_url(self) -> str:
        if self.__events_queue_url is None:
            self.__events_queue_url = self.__cloudformation_stack.get_physical_resource_id_for('EventsQueue')

        return self.__events_queue_url

    @property
    def results_table_name(self) -> str:
        if self.__results_table_name is None:
            self.__results_table_name = self.__cloudformation_stack.get_physical_resource_id_for('ResultsTable')

        return self.__results_table_name

    @property
    def test_context_bucket(self) -> S3Bucket:
        if self.__test_context_bucket is None:
            self.__test_context_bucket = self.get_s3_bucket('TestContext')

        return self.__test_context_bucket



import threading
import traceback


def handle_uncaught_thread_exception(args):
    print('Uncaught exception in thread')
    print(f"Exception Type: {args.exc_type.__name__}")
    print(f"Exception Message: {args.exc_value}")
    traceback.print_tb(args.exc_traceback)


threading.excepthook = handle_uncaught_thread_exception



import json
from uuid import uuid4

from boto3 import Session

from aws_test_harness.state_machine_execution import StateMachineExecution


class StateMachine:
    def __init__(self, arn: str, boto_session: Session):
        self.__arn = arn
        self.__sfn_client = boto_session.client('stepfunctions')

    def execute(self, execution_input, timeout_seconds: float = 30):
        execution = self.start_execution(execution_input)
        execution.wait_for_completion(timeout_seconds)

        return execution

    def start_execution(self, execution_input):
        response = self.__sfn_client.start_execution(
            stateMachineArn=self.__arn,
            input=json.dumps(execution_input),
            name=f"test-{uuid4()}"
        )

        return StateMachineExecution(response["executionArn"], self.__sfn_client)

    def send_task_success(self, task_token: str, output: object) -> None:
        self.__sfn_client.send_task_success(
            taskToken=task_token,
            output=json.dumps(output)
        )

    def send_task_failure(self, task_token: str, error: str, cause: str) -> None:
        self.__sfn_client.send_task_failure(
            taskToken=task_token,
            error=error,
            cause=cause
        )



from typing import Mapping

from boto3 import Session
from mypy_boto3_dynamodb.service_resource import DynamoDBServiceResource, Table
from mypy_boto3_dynamodb.type_defs import TableAttributeValueTypeDef


class DynamoDBTable:
    def __init__(self, name: str, session: Session):
        self.__name = name
        dynamodb_resource: DynamoDBServiceResource = session.resource('dynamodb')
        self.__table: Table = dynamodb_resource.Table(name)
        self.__scan_paginator = dynamodb_resource.meta.client.get_paginator('scan')

    @property
    def name(self):
        return self.__name

    def get_item(self, key: Mapping[str, TableAttributeValueTypeDef]):
        result = self.__table.get_item(Key=key)
        return result.get('Item')

    def put_item(self, item: Mapping[str, TableAttributeValueTypeDef]):
        self.__table.put_item(Item=item)

    def delete_item(self, key: Mapping[str, TableAttributeValueTypeDef]):
        self.__table.delete_item(Key=key)

    def empty(self):
        attribute_names = [key['AttributeName'] for key in self.__table.key_schema]

        page_iterator = self.__scan_paginator.paginate(
            TableName=self.__name,
            ProjectionExpression=','.join(attribute_names)
        )

        with self.__table.batch_writer() as batch:
            for item in page_iterator.search('Items'):
                key = dict((attribute_name, item[attribute_name]) for attribute_name in attribute_names)
                batch.delete_item(Key=key)
